@@ -4,7 +4,9 @@ from dataclasses import dataclass, field, fields
 from enum import IntEnum, StrEnum
 from typing import Any
 
-from roborock import RoborockProductNickname
+from roborock.data.code_mappings import RoborockProductNickname
+from roborock.data.containers import RoborockBase
+from roborock.data.v1 import RoborockDockTypeCode
 
 
 class NewFeatureStrBit(IntEnum):
@@ -57,6 +59,35 @@ class NewFeatureStrBit(IntEnum):
     MIDWAY_BACK_TO_DOCK = 85
     SUPPORT_MAIN_BRUSH_UP_DOWN = 86
     EGG_DANCE_MODE = 87
+    MECHANICAL_ARM_MODE = 89
+    TIDYUP_ZONES = MECHANICAL_ARM_MODE
+    CLEAN_TIME_LINE = 91
+    CLEAN_THEN_MOP_MODE = 93
+    TYPE_IDENTIFY = 94
+    SUPPORT_GET_PARTICULAR_STATUS = 96
+    THREE_D_MAPPING_INNER_TEST = 97
+    SYNC_SERVER_NAME = 98
+    SHOULD_SHOW_ARM_OVER_LOAD = 99
+    COLLECT_DUST_COUNT_SHOW = 100
+    SUPPORT_API_APP_STOP_GRASP = 101
+    CTM_WITH_REPEAT = 102
+    SIDE_BRUSH_LIFT_CARPET = 104
+    DETECT_WIRE_CARPET = 105
+    WATER_SLIDE_MODE = 106
+    SOAK_AND_WASH = 107
+    CLEAN_EFFICIENCY = 108
+    BACK_WASH_NEW_SMART = 109
+    DUAL_BAND_WI_FI = 110
+    PROGRAM_MODE = 111
+    CLEAN_FLUID_DELIVERY = 112
+    CARPET_LONG_HAIRED_EX = 113
+    OVER_SEA_CTM = 114
+    FULL_DUPLES_SWITCH = 115
+    LOW_AREA_ACCESS = 116
+    FOLLOW_LOW_OBS = 117
+    TWO_GEARS_NO_COLLISION = 118
+    CARPET_SHAPE_TYPE = 119
+    SR_MAP = 120
 
 
 class ProductFeatures(StrEnum):
@@ -116,14 +147,45 @@ DUAL_LINE_CAMERA_FEATURES = [
 NEW_DEFAULT_FEATURES = [ProductFeatures.REMOTE_BACK, ProductFeatures.CLEANMODE_MAXPLUS]
 
 
-PEARL_FEATURES = NEW_DEFAULT_FEATURES + SINGLE_LINE_CAMERA_FEATURES + [ProductFeatures.MOP_SPIN_MODULE]
+PEARL_FEATURES = SINGLE_LINE_CAMERA_FEATURES + [ProductFeatures.CLEANMODE_MAXPLUS, ProductFeatures.MOP_SPIN_MODULE]
 PEARL_PLUS_FEATURES = NEW_DEFAULT_FEATURES + RGB_CAMERA_FEATURES + [ProductFeatures.MOP_SPIN_MODULE]
 ULTRON_FEATURES = NEW_DEFAULT_FEATURES + DUAL_LINE_CAMERA_FEATURES + [ProductFeatures.MOP_SHAKE_MODULE]
 ULTRONSV_FEATURES = NEW_DEFAULT_FEATURES + RGB_CAMERA_FEATURES + [ProductFeatures.MOP_SHAKE_MODULE]
-TANOSS_FEATURES = NEW_DEFAULT_FEATURES + [ProductFeatures.MOP_SHAKE_MODULE]
-TOPAZSPOWER_FEATURES = NEW_DEFAULT_FEATURES + [ProductFeatures.MOP_SHAKE_MODULE]
+TANOSS_FEATURES = [ProductFeatures.REMOTE_BACK, ProductFeatures.MOP_SHAKE_MODULE]
+TOPAZSPOWER_FEATURES = [ProductFeatures.CLEANMODE_MAXPLUS, ProductFeatures.MOP_SHAKE_MODULE]
 
-product_feature_map = {
+PRODUCTS_WITHOUT_CUSTOM_CLEAN: set[RoborockProductNickname] = {
+    RoborockProductNickname.TANOS,
+    RoborockProductNickname.RUBYPLUS,
+    RoborockProductNickname.RUBYSC,
+    RoborockProductNickname.RUBYSE,
+}
+PRODUCTS_WITHOUT_DEFAULT_3D_MAP: set[RoborockProductNickname] = {
+    RoborockProductNickname.TANOS,
+    RoborockProductNickname.TANOSSPLUS,
+    RoborockProductNickname.TANOSE,
+    RoborockProductNickname.TANOSV,
+    RoborockProductNickname.RUBYPLUS,
+    RoborockProductNickname.RUBYSC,
+    RoborockProductNickname.RUBYSE,
+}
+PRODUCTS_WITHOUT_PURE_CLEAN_MOP: set[RoborockProductNickname] = {
+    RoborockProductNickname.TANOS,
+    RoborockProductNickname.TANOSE,
+    RoborockProductNickname.TANOSV,
+    RoborockProductNickname.TANOSSLITE,
+    RoborockProductNickname.TANOSSE,
+    RoborockProductNickname.TANOSSC,
+    RoborockProductNickname.ULTRONLITE,
+    RoborockProductNickname.ULTRONE,
+    RoborockProductNickname.RUBYPLUS,
+    RoborockProductNickname.RUBYSLITE,
+    RoborockProductNickname.RUBYSC,
+    RoborockProductNickname.RUBYSE,
+}
+
+# Base map containing the initial, unconditional features for each product.
+_BASE_PRODUCT_FEATURE_MAP: dict[RoborockProductNickname, list[ProductFeatures]] = {
     RoborockProductNickname.PEARL: PEARL_FEATURES,
     RoborockProductNickname.PEARLS: PEARL_FEATURES,
     RoborockProductNickname.PEARLPLUS: PEARL_PLUS_FEATURES,
@@ -139,7 +201,8 @@ product_feature_map = {
     RoborockProductNickname.PEARLSLITE: PEARL_FEATURES,
     RoborockProductNickname.PEARLE: PEARL_FEATURES,
     RoborockProductNickname.PEARLELITE: PEARL_FEATURES,
-    RoborockProductNickname.VIVIANC: PEARL_PLUS_FEATURES,
+    RoborockProductNickname.VIVIANC: [ProductFeatures.CLEANMODE_MAXPLUS, ProductFeatures.MOP_SPIN_MODULE]
+    + SINGLE_LINE_CAMERA_FEATURES,
     RoborockProductNickname.CORALPRO: PEARL_PLUS_FEATURES,
     RoborockProductNickname.ULTRONLITE: SINGLE_LINE_CAMERA_FEATURES
     + [ProductFeatures.CLEANMODE_NONE_PURECLEANMOP_WITH_MAXPLUS, ProductFeatures.MOP_ELECTRONIC_MODULE],
@@ -150,7 +213,7 @@ product_feature_map = {
     ],
     RoborockProductNickname.ULTRONSPLUS: ULTRON_FEATURES,
     RoborockProductNickname.VERDELITE: ULTRONSV_FEATURES,
-    RoborockProductNickname.TOPAZS: NEW_DEFAULT_FEATURES + [ProductFeatures.MOP_SHAKE_MODULE],
+    RoborockProductNickname.TOPAZS: [ProductFeatures.REMOTE_BACK, ProductFeatures.MOP_SHAKE_MODULE],
     RoborockProductNickname.TOPAZSPLUS: NEW_DEFAULT_FEATURES
     + DUAL_LINE_CAMERA_FEATURES
     + [ProductFeatures.MOP_SHAKE_MODULE],
@@ -173,9 +236,19 @@ product_feature_map = {
     RoborockProductNickname.RUBYSLITE: [ProductFeatures.MOP_ELECTRONIC_MODULE],
 }
 
+PRODUCT_FEATURE_MAP: dict[RoborockProductNickname, list[ProductFeatures]] = {
+    product: (
+        features
+        + ([ProductFeatures.DEFAULT_CLEANMODECUSTOM] if product not in PRODUCTS_WITHOUT_CUSTOM_CLEAN else [])
+        + ([ProductFeatures.DEFAULT_MAP3D] if product not in PRODUCTS_WITHOUT_DEFAULT_3D_MAP else [])
+        + ([ProductFeatures.CLEANMODE_PURECLEANMOP] if product not in PRODUCTS_WITHOUT_PURE_CLEAN_MOP else [])
+    )
+    for product, features in _BASE_PRODUCT_FEATURE_MAP.items()
+}
+
 
 @dataclass
-class DeviceFeatures:
+class DeviceFeatures(RoborockBase):
     """Represents the features supported by a Roborock device."""
 
     # Features from robot_new_features (lower 32 bits)
@@ -240,7 +313,7 @@ class DeviceFeatures:
     is_support_incremental_map: bool = field(metadata={"new_feature_str_mask": (4194304, 8)})
     is_offline_map_supported: bool = field(metadata={"new_feature_str_mask": (16384, 8)})
     is_super_deep_wash_supported: bool = field(metadata={"new_feature_str_mask": (32768, 8)})
-    is_ces2022_supported: bool = field(metadata={"new_feature_str_mask": (65536, 8)})
+    is_ces_2022_supported: bool = field(metadata={"new_feature_str_mask": (65536, 8)})
     is_dss_believable: bool = field(metadata={"new_feature_str_mask": (131072, 8)})
     is_main_brush_up_down_supported_from_str: bool = field(metadata={"new_feature_str_mask": (262144, 8)})
     is_goto_pure_clean_path_supported: bool = field(metadata={"new_feature_str_mask": (524288, 8)})
@@ -349,6 +422,59 @@ class DeviceFeatures:
         metadata={"new_feature_str_bit": NewFeatureStrBit.SUPPORT_MAIN_BRUSH_UP_DOWN}
     )
     is_egg_dance_mode_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.EGG_DANCE_MODE})
+    is_mechanical_arm_mode_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.MECHANICAL_ARM_MODE}
+    )
+    is_tidyup_zones_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.TIDYUP_ZONES})
+    is_clean_time_line_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.CLEAN_TIME_LINE})
+    is_clean_then_mop_mode_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.CLEAN_THEN_MOP_MODE}
+    )
+    is_type_identify_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.TYPE_IDENTIFY})
+    is_support_get_particular_status_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.SUPPORT_GET_PARTICULAR_STATUS}
+    )
+    is_three_d_mapping_inner_test_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.THREE_D_MAPPING_INNER_TEST}
+    )
+    is_sync_server_name_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.SYNC_SERVER_NAME})
+    is_should_show_arm_over_load_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.SHOULD_SHOW_ARM_OVER_LOAD}
+    )
+    is_collect_dust_count_show_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.COLLECT_DUST_COUNT_SHOW}
+    )
+    is_support_api_app_stop_grasp_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.SUPPORT_API_APP_STOP_GRASP}
+    )
+    is_ctm_with_repeat_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.CTM_WITH_REPEAT})
+    is_side_brush_lift_carpet_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.SIDE_BRUSH_LIFT_CARPET}
+    )
+    is_detect_wire_carpet_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.DETECT_WIRE_CARPET})
+    is_water_slide_mode_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.WATER_SLIDE_MODE})
+    is_soak_and_wash_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.SOAK_AND_WASH})
+    is_clean_efficiency_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.CLEAN_EFFICIENCY})
+    is_back_wash_new_smart_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.BACK_WASH_NEW_SMART}
+    )
+    is_dual_band_wi_fi_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.DUAL_BAND_WI_FI})
+    is_program_mode_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.PROGRAM_MODE})
+    is_clean_fluid_delivery_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.CLEAN_FLUID_DELIVERY}
+    )
+    is_carpet_long_haired_ex_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.CARPET_LONG_HAIRED_EX}
+    )
+    is_over_sea_ctm_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.OVER_SEA_CTM})
+    is_full_duples_switch_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.FULL_DUPLES_SWITCH})
+    is_low_area_access_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.LOW_AREA_ACCESS})
+    is_follow_low_obs_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.FOLLOW_LOW_OBS})
+    is_two_gears_no_collision_supported: bool = field(
+        metadata={"new_feature_str_bit": NewFeatureStrBit.TWO_GEARS_NO_COLLISION}
+    )
+    is_carpet_shape_type_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.CARPET_SHAPE_TYPE})
+    is_sr_map_supported: bool = field(metadata={"new_feature_str_bit": NewFeatureStrBit.SR_MAP})
 
     # Features from feature_info list
     is_led_status_switch_supported: bool = field(metadata={"robot_features": 119})
@@ -423,6 +549,10 @@ class DeviceFeatures:
     is_clean_route_setting_supported: bool = field(
         metadata={"product_features": [ProductFeatures.MOP_SHAKE_MODULE, ProductFeatures.MOP_SPIN_MODULE]}
     )
+    is_mop_shake_module_supported: bool = field(metadata={"product_features": [ProductFeatures.MOP_SHAKE_MODULE]})
+    is_customized_clean_supported: bool = field(
+        metadata={"product_features": [ProductFeatures.MOP_SHAKE_MODULE, ProductFeatures.MOP_SPIN_MODULE]}
+    )
 
     @classmethod
     def from_feature_flags(
@@ -492,7 +622,7 @@ class DeviceFeatures:
                     kwargs[f.name] = product_nickname not in blacklist
             elif (product_features := f.metadata.get("product_features")) is not None:
                 if product_nickname is not None:
-                    available_features = product_feature_map.get(product_nickname, [])
+                    available_features = PRODUCT_FEATURE_MAP.get(product_nickname, [])
                     if any(feat in available_features for feat in product_features):  # type: ignore
                         kwargs[f.name] = True
 
@@ -501,3 +631,25 @@ class DeviceFeatures:
     def get_supported_features(self) -> list[str]:
         """Returns a list of supported features (Primarily used for logging purposes)."""
         return [k for k, v in vars(self).items() if v]
+
+
+WASH_N_FILL_DOCK_TYPES = [
+    RoborockDockTypeCode.empty_wash_fill_dock,
+    RoborockDockTypeCode.s8_dock,
+    RoborockDockTypeCode.p10_dock,
+    RoborockDockTypeCode.p10_pro_dock,
+    RoborockDockTypeCode.s8_maxv_ultra_dock,
+    RoborockDockTypeCode.qrevo_s_dock,
+    RoborockDockTypeCode.saros_r10_dock,
+    RoborockDockTypeCode.qrevo_curv_dock,
+]
+
+
+def is_wash_n_fill_dock(dock_type: RoborockDockTypeCode) -> bool:
+    """Check if the dock type is a wash and fill dock."""
+    return dock_type in WASH_N_FILL_DOCK_TYPES
+
+
+def is_valid_dock(dock_type: RoborockDockTypeCode) -> bool:
+    """Check if device supports a dock."""
+    return dock_type != RoborockDockTypeCode.no_dock
